"""Forms definitions"""

from django import forms
from django.utils.translation import gettext as _
from netbox.forms import (
    NetBoxModelForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
)
from utilities.forms.fields import DynamicModelMultipleChoiceField, TagFilterField
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from .models import (
    Mapping,
    AuthenticationChoices,
    HttpHeader,
    ApplyToChoices,
    SamlConfig,
)


class MappingForm(NetBoxModelForm):
    """Mapping form definition class"""

    class Meta:
        model = Mapping
        fields = (
            "source",
            "target",
            "authentication",
            "webdav",
            "testingpage",
            "Comment",
            "tags",
        )
        help_texts = {"target": "URL-friendly unique shorthand"}
        labels = {
            "source": "Source",
            "target": "Target",
        }


class MappingFilterForm(NetBoxModelFilterSetForm):
    """Mapping filter form definition class"""

    model = Mapping
    source = forms.CharField(
        max_length=120, min_length=1, required=False, label="Source URL"
    )
    target = forms.CharField(
        max_length=120, min_length=1, required=False, label="Target URL"
    )
    authentication = forms.MultipleChoiceField(
        choices=AuthenticationChoices,
        required=False,
    )
    webdav = forms.BooleanField(
        required=False, widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES)
    )
    testingpage = forms.CharField(
        max_length=120, min_length=1, required=False, label="Testing URL"
    )
    Comment = forms.CharField(
        max_length=500, min_length=1, required=False, label="Comment"
    )
    tag = TagFilterField(model)


class MappingImportForm(NetBoxModelImportForm):
    """Mapping importation form definition class"""

    class Meta:
        model = Mapping
        fields = ("source", "target", "authentication", "testingpage", "webdav")


class HttpHeaderForm(NetBoxModelForm):
    """HTTP header form definition class"""

    class Meta:
        model = HttpHeader
        fields = ("mapping", "name", "value", "apply_to")
        labels = {
            "mapping": "Mapping",
            "name": "Name",
            "value": "Value",
            "apply_to": "Apply to",
        }


class HttpHeaderFilterForm(NetBoxModelFilterSetForm):
    """HTTP header filter form definition class"""

    model = HttpHeader
    name = forms.CharField(
        max_length=120, min_length=1, required=False, label="Header name"
    )
    value = forms.CharField(
        max_length=256, min_length=1, required=False, label="Header value"
    )
    apply_to = forms.ChoiceField(
        choices=add_blank_choice(ApplyToChoices), required=False, label="Apply to"
    )
    mapping = DynamicModelMultipleChoiceField(
        queryset=Mapping.objects.all(), required=False, label=_("Mapping")
    )
    tag = TagFilterField(model)


class SamlConfigForm(NetBoxModelForm):
    """SAML config form definition class"""

    class Meta:
        model = SamlConfig
        fields = ("mapping", "acs_url", "logout_url", "force_nauth")
        labels = {
            "mapping": "Mapping",
            "acs_url": "ACS URL",
            "logout_url": "Logout URL",
            "force_nauth": "Force AuthnRequest",
        }
