"""Models definitions"""

from django.db import models
from django.urls import reverse
from django.core.validators import URLValidator
from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet

URL_MAX_SIZE = 2000


class AuthenticationChoices(ChoiceSet):
    """Authentication choices definition class"""

    key = "Mapping.authentication"

    DEFAULT_VALUE = "none"

    CHOICES = [
        ("none", "None", "dark"),
        ("ldap", "Ldap", "blue"),
        ("ecas", "Ecas", "blue"),
    ]


class ApplyToChoices(ChoiceSet):
    """Apply to choices definition class"""

    key = "HttpHeader.applyTo"

    DEFAULT_VALUE = "request"

    CHOICES = [
        ("request", "Request", "blue"),
        ("response", "Response", "red"),
    ]


class Mapping(NetBoxModel):
    """Mapping definition class"""

    source = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Source",
        validators=[URLValidator(message="It must be a url")],
        unique=True,
    )
    target = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Target",
        validators=[URLValidator(message="It must be a url")],
    )
    authentication = models.CharField(
        max_length=30,
        choices=AuthenticationChoices,
        default=AuthenticationChoices.DEFAULT_VALUE,
        blank=False,
        verbose_name="Auth",
    )
    testingpage = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=True,
        null=True,
        validators=[URLValidator(message="It must be a url")],
    )
    webdav = models.BooleanField(
        default=False,
    )
    Comment = models.CharField(max_length=500, blank=True)

    class Meta:
        ordering = ("source", "target")

    def __str__(self):
        return f"{self.source}"

    def get_absolute_url(self):
        """override"""
        return reverse("plugins:netbox_rps_plugin:mapping", args=[self.pk])


class SamlConfig(NetBoxModel):
    """SAML config definition class"""

    acs_url = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="ACS URL",
        validators=[URLValidator(message="It must be a url")],
    )
    logout_url = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Logout URL",
        validators=[URLValidator(message="It must be a url")],
    )
    force_nauth = models.BooleanField(
        default=False,
    )
    Mapping = models.OneToOneField(
        Mapping,
        on_delete=models.CASCADE,
        related_name="saml_config",
        db_column="mapping_id",
        name="mapping",
    )


class HttpHeader(NetBoxModel):
    """HTTP Header definition class"""

    mapping = models.ForeignKey(
        Mapping, on_delete=models.CASCADE, related_name="http_headers"
    )
    name = models.CharField(max_length=120, blank=False, verbose_name="Header name")
    value = models.CharField(
        max_length=256, null=True, blank=True, verbose_name="Header value"
    )
    apply_to = models.CharField(
        max_length=30,
        choices=ApplyToChoices,
        default=ApplyToChoices.DEFAULT_VALUE,
        blank=False,
        verbose_name="Apply to",
    )

    class Meta:
        unique_together = ["mapping", "name", "apply_to"]
        ordering = ["name"]

    def __str__(self):
        return f"{self.name}"

    def get_absolute_url(self):
        """override"""
        return reverse("plugins:netbox_rps_plugin:httpheader", args=[self.pk])
